from __future__ import unicode_literals

__title__ = "dj-stripe"
__summary__ = "Django + Stripe for humans "
__uri__ = "https://github.com/pydanny/dj-stripe/"

__version__ = "0.2.2"

__author__ = "Daniel Greenfeld"
__email__ = "pydanny@gmail.com"

__license__ = "BSD"
__copyright__ = "Copyright 2013 Daniel Greenfeld"
